@@ -35,6 +35,8 @@
 #include <netdb.h>
 #include <pwd.h>
 #include <termios.h>
+
+#include <semaphore.h>
 #include <pthread.h>
 
 #if __sun
@@ -58,6 +60,7 @@ typedef pthread_once_t uv_once_t;
 typedef pthread_t uv_thread_t;
 typedef pthread_mutex_t uv_mutex_t;
 typedef pthread_rwlock_t uv_rwlock_t;
+typedef sem_t uv_sem_t;
 
 /* Platform-specific definitions for uv_spawn support. */
 typedef gid_t uv_gid_t;
