@@ -19,12 +19,13 @@
  * IN THE SOFTWARE.
  */
 
+#include <assert.h>
+#include <io.h>
 
 #include "uv.h"
 #include "internal.h"
-
-#include <assert.h>
-#include <io.h>
+#include "handle-inl.h"
+#include "req-inl.h"
 
 
 static const GUID uv_msafd_provider_ids[UV_MSAFD_PROVIDER_COUNT] = {
@@ -186,6 +187,7 @@ static void uv__fast_poll_process_poll_req(uv_loop_t* loop, uv_poll_t* handle,
     if (afd_poll_info->Handles[0].Events & AFD_POLL_LOCAL_CLOSE) {
       /* Stop polling. */
       handle->events = 0;
+      uv__handle_stop(handle);
     }
 
     if (events != 0) {
