@@ -2,47 +2,6 @@
 static PyObject* PyExc_AsyncError;
 
 
-static void
-on_async_close(uv_handle_t *handle)
-{
-    PyGILState_STATE gstate = PyGILState_Ensure();
-    Async *self;
-    PyObject *result;
-
-    ASSERT(handle);
-
-    self = (Async *)handle->data;
-    ASSERT(self);
-
-    if (self->on_close_cb != Py_None) {
-        result = PyObject_CallFunctionObjArgs(self->on_close_cb, self, NULL);
-        if (result == NULL) {
-            PyErr_WriteUnraisable(self->on_close_cb);
-        }
-        Py_XDECREF(result);
-    }
-
-    handle->data = NULL;
-    PyMem_Free(handle);
-
-    /* Refcount was increased in func_close */
-    Py_DECREF(self);
-
-    PyGILState_Release(gstate);
-}
-
-
-static void
-on_async_dealloc_close(uv_handle_t *handle)
-{
-    PyGILState_STATE gstate = PyGILState_Ensure();
-    ASSERT(handle);
-    handle->data = NULL;
-    PyMem_Free(handle);
-    PyGILState_Release(gstate);
-}
-
-
 static void
 on_async_callback(uv_async_t *async, int status)
 {
@@ -70,70 +29,18 @@ on_async_callback(uv_async_t *async, int status)
 
 
 static PyObject *
-Async_func_send(Async *self, PyObject *args)
+Async_func_send(Async *self)
 {
     int r;
-    PyObject *tmp, *callback;
 
-    tmp = NULL;
+    RAISE_IF_HANDLE_CLOSED(self, PyExc_HandleClosedError, NULL);
 
-    if (!self->uv_handle) {
-        PyErr_SetString(PyExc_AsyncError, "async is closed");
-        return NULL;
-    }
-
-    if (!PyArg_ParseTuple(args, "O:send", &callback)) {
-        return NULL;
-    }
-
-    if (!PyCallable_Check(callback)) {
-        PyErr_SetString(PyExc_TypeError, "a callable is required");
-        return NULL;
-    }
-
-    r = uv_async_send(self->uv_handle);
+    r = uv_async_send((uv_async_t *)UV_HANDLE(self));
     if (r != 0) {
-        raise_uv_exception(self->loop, PyExc_AsyncError);
+        RAISE_UV_EXCEPTION(UV_LOOP((Handle *)self), PyExc_AsyncError);
         return NULL;
     }
 
-    tmp = self->callback;
-    Py_INCREF(callback);
-    self->callback = callback;
-    Py_XDECREF(tmp);
-
-    Py_RETURN_NONE;
-}
-
-
-static PyObject *
-Async_func_close(Async *self, PyObject *args)
-{
-    PyObject *callback = Py_None;
-
-    if (!self->uv_handle) {
-        PyErr_SetString(PyExc_AsyncError, "Async is already closed");
-        return NULL;
-    }
-
-    if (!PyArg_ParseTuple(args, "|O:close", &callback)) {
-        return NULL;
-    }
-
-    if (callback != Py_None && !PyCallable_Check(callback)) {
-        PyErr_SetString(PyExc_TypeError, "a callable or None is required");
-        return NULL;
-    }
-
-    Py_INCREF(callback);
-    self->on_close_cb = callback;
-
-    /* Increase refcount so that object is not removed before the callback is called */
-    Py_INCREF(self);
-
-    uv_close((uv_handle_t *)self->uv_handle, on_async_close);
-    self->uv_handle = NULL;
-
     Py_RETURN_NONE;
 }
 
@@ -144,22 +51,28 @@ Async_tp_init(Async *self, PyObject *args, PyObject *kwargs)
     int r;
     uv_async_t *uv_async = NULL;
     Loop *loop;
+    PyObject *callback;
     PyObject *tmp = NULL;
 
     UNUSED_ARG(kwargs);
 
-    if (self->uv_handle) {
+    if (UV_HANDLE(self)) {
         PyErr_SetString(PyExc_AsyncError, "Object already initialized");
         return -1;
     }
 
-    if (!PyArg_ParseTuple(args, "O!:__init__", &LoopType, &loop)) {
+    if (!PyArg_ParseTuple(args, "O!O:__init__", &LoopType, &loop, &callback)) {
         return -1;
     }
 
-    tmp = (PyObject *)self->loop;
+    if (!PyCallable_Check(callback)) {
+        PyErr_SetString(PyExc_TypeError, "a callable is required");
+        return -1;
+    }
+
+    tmp = (PyObject *)((Handle *)self)->loop;
     Py_INCREF(loop);
-    self->loop = loop;
+    ((Handle *)self)->loop = loop;
     Py_XDECREF(tmp);
 
     uv_async = PyMem_Malloc(sizeof(uv_async_t));
@@ -169,14 +82,20 @@ Async_tp_init(Async *self, PyObject *args, PyObject *kwargs)
         return -1;
     }
 
-    r = uv_async_init(UV_LOOP(self), uv_async, on_async_callback);
+    r = uv_async_init(UV_HANDLE_LOOP(self), uv_async, on_async_callback);
     if (r != 0) {
-        raise_uv_exception(self->loop, PyExc_AsyncError);
+        RAISE_UV_EXCEPTION(UV_HANDLE_LOOP(self), PyExc_AsyncError);
         Py_DECREF(loop);
         return -1;
     }
+
+    tmp = self->callback;
+    Py_INCREF(callback);
+    self->callback = callback;
+    Py_XDECREF(tmp);
+
     uv_async->data = (void *)self;
-    self->uv_handle = uv_async;
+    UV_HANDLE(self) = (uv_handle_t *)uv_async;
 
     return 0;
 }
@@ -185,11 +104,10 @@ Async_tp_init(Async *self, PyObject *args, PyObject *kwargs)
 static PyObject *
 Async_tp_new(PyTypeObject *type, PyObject *args, PyObject *kwargs)
 {
-    Async *self = (Async *)PyType_GenericNew(type, args, kwargs);
+    Async *self = (Async *)HandleType.tp_new(type, args, kwargs);
     if (!self) {
         return NULL;
     }
-    self->uv_handle = NULL;
     return (PyObject *)self;
 }
 
@@ -197,10 +115,8 @@ Async_tp_new(PyTypeObject *type, PyObject *args, PyObject *kwargs)
 static int
 Async_tp_traverse(Async *self, visitproc visit, void *arg)
 {
-    Py_VISIT(self->data);
     Py_VISIT(self->callback);
-    Py_VISIT(self->on_close_cb);
-    Py_VISIT(self->loop);
+    HandleType.tp_traverse((PyObject *)self, visit, arg);
     return 0;
 }
 
@@ -208,47 +124,25 @@ Async_tp_traverse(Async *self, visitproc visit, void *arg)
 static int
 Async_tp_clear(Async *self)
 {
-    Py_CLEAR(self->data);
     Py_CLEAR(self->callback);
-    Py_CLEAR(self->on_close_cb);
-    Py_CLEAR(self->loop);
+    HandleType.tp_clear((PyObject *)self);
     return 0;
 }
 
 
-static void
-Async_tp_dealloc(Async *self)
-{
-    if (self->uv_handle) {
-        uv_close((uv_handle_t *)self->uv_handle, on_async_dealloc_close);
-        self->uv_handle = NULL;
-    }
-    Async_tp_clear(self);
-    Py_TYPE(self)->tp_free((PyObject *)self);
-}
-
-
 static PyMethodDef
 Async_tp_methods[] = {
-    { "send", (PyCFunction)Async_func_send, METH_VARARGS, "Send the Async signal." },
-    { "close", (PyCFunction)Async_func_close, METH_VARARGS, "Close this Async handle." },
+    { "send", (PyCFunction)Async_func_send, METH_NOARGS, "Send the Async signal." },
     { NULL }
 };
 
 
-static PyMemberDef Async_tp_members[] = {
-    {"loop", T_OBJECT_EX, offsetof(Async, loop), READONLY, "Loop where this Async is running on."},
-    {"data", T_OBJECT, offsetof(Async, data), 0, "Arbitrary data."},
-    {NULL}
-};
-
-
 static PyTypeObject AsyncType = {
     PyVarObject_HEAD_INIT(NULL, 0)
     "pyuv.Async",                                                   /*tp_name*/
     sizeof(Async),                                                  /*tp_basicsize*/
     0,                                                              /*tp_itemsize*/
-    (destructor)Async_tp_dealloc,                                   /*tp_dealloc*/
+    0,                                                              /*tp_dealloc*/
     0,                                                              /*tp_print*/
     0,                                                              /*tp_getattr*/
     0,                                                              /*tp_setattr*/
@@ -272,7 +166,7 @@ static PyTypeObject AsyncType = {
     0,                                                              /*tp_iter*/
     0,                                                              /*tp_iternext*/
     Async_tp_methods,                                               /*tp_methods*/
-    Async_tp_members,                                               /*tp_members*/
+    0,                                                              /*tp_members*/
     0,                                                              /*tp_getsets*/
     0,                                                              /*tp_base*/
     0,                                                              /*tp_dict*/
