@@ -1,7 +1,9 @@
 #!/usr/bin/env python
 
+from __future__ import print_function
+
 import sys
 
-print "TEST"
+print("TEST")
 sys.stdout.flush()
 
